@@ -88,21 +88,30 @@ myfgets(char *buf, int maxlen, FILE *fp)
  * </EN>
  */
 static boolean
-read_grammar(FILE *fp, DFA_INFO **ret_dfa, WORD_INFO **ret_winfo, HTK_HMM_INFO *hmminfo)
+read_grammar(FILE *fp, DFA_INFO **ret_dfa, WORD_INFO **ret_winfo, HTK_HMM_INFO *hmminfo, RecogProcess *r)
 {
-  DFA_INFO *dfa;
+  DFA_INFO *dfa = NULL;
   WORD_INFO *winfo;
+  JCONF_LM *lmconf;
 
   /* load grammar: dfa and dict in turn */
-  dfa = dfa_info_new();
-  if (!rddfa_fp(fp, dfa)) {
-    return FALSE;
+  if (r->lmvar != LM_DFA_WORD) {
+    dfa = dfa_info_new();
+    if (!rddfa_fp(fp, dfa)) {
+      return FALSE;
+    }
   }
   winfo = word_info_new();
-  /* ignore MONOTREE */
-  if (!voca_load_htkdict_fp(fp, winfo, hmminfo, FALSE)) {
-    dfa_info_free(dfa);
-    return FALSE;
+  if (r->lmvar == LM_DFA_WORD) {
+    lmconf = r->lm->config;
+    if (!voca_load_wordlist_fp(fp, winfo, hmminfo, lmconf->wordrecog_head_silence_model_name, lmconf->wordrecog_tail_silence_model_name, (lmconf->wordrecog_silence_context_name[0] == '\0') ? NULL : lmconf->wordrecog_silence_context_name)) {
+      return FALSE;
+    }
+  } else {
+    if (!voca_load_htkdict_fp(fp, winfo, hmminfo, FALSE)) {
+      dfa_info_free(dfa);
+      return FALSE;
+    }
   }
   *ret_dfa = dfa;
   *ret_winfo = winfo;
@@ -246,7 +255,7 @@ msock_exec_command(char *command, Recog *recog)
       p = NULL;
     }
     /* read a new grammar via socket */
-    if (read_grammar(module_fp, &new_dfa, &new_winfo, cur->am->hmminfo) == FALSE) {
+    if (read_grammar(module_fp, &new_dfa, &new_winfo, cur->am->hmminfo, cur) == FALSE) {
       module_send(module_sd, "<GRAMMAR STATUS=\"ERROR\" REASON=\"WRONG DATA\"/>\n.\n");
     } else {
       if (cur->lmtype == LM_DFA) {
@@ -280,7 +289,7 @@ msock_exec_command(char *command, Recog *recog)
       p = NULL;
     }
     /* read a new grammar via socket */
-    if (read_grammar(module_fp, &new_dfa, &new_winfo, cur->am->hmminfo) == FALSE) {
+    if (read_grammar(module_fp, &new_dfa, &new_winfo, cur->am->hmminfo, cur) == FALSE) {
       module_send(module_sd, "<GRAMMAR STATUS=\"ERROR\" REASON=\"WRONG DATA\"/>\n.\n");
     } else {
       if (cur->lmtype == LM_DFA) {
@@ -315,6 +324,7 @@ msock_exec_command(char *command, Recog *recog)
 	q = p;
 	while(*q != '\0' && *q != '\r' && *q != '\n') {
 	  if (*q < '0' || *q > '9') break;
+	  q++;
 	}
 	if (*q == '\0' || *q == '\r' || *q == '\n') { /* numeric */
 	  gid = atoi(p);
@@ -349,6 +359,7 @@ msock_exec_command(char *command, Recog *recog)
 	q = p;
 	while(*q != '\0' && *q != '\r' && *q != '\n') {
 	  if (*q < '0' || *q > '9') break;
+	  q++;
 	}
 	if (*q == '\0' || *q == '\r' || *q == '\n') { /* numeric */
 	  gid = atoi(p);
@@ -385,6 +396,7 @@ msock_exec_command(char *command, Recog *recog)
 	q = p;
 	while(*q != '\0' && *q != '\r' && *q != '\n') {
 	  if (*q < '0' || *q > '9') break;
+	  q++;
 	}
 	if (*q == '\0' || *q == '\r' || *q == '\n') { /* numeric */
 	  gid = atoi(p);
