@@ -12,7 +12,7 @@
  * @author Akinobu Lee
  * @date   Tue Sep 06 17:18:46 2005
  *
- * $Revision: 1.5 $
+ * $Revision: 1.7 $
  * 
  */
 /*
@@ -26,9 +26,6 @@
 
 extern boolean separate_score_flag;
 
-/// Grammar status to be processed in the next reload timing.
-static char *hookstr[] = {"", "delete", "activate", "deactivate"};
-
 static boolean have_progout = FALSE;
 
 /* for short pause segmentation and successive decoding */
@@ -759,6 +756,8 @@ result_pass2(Recog *recog, void *dummy)
   Sentence *s;
   RecogProcess *r;
   boolean multi;
+  HMM_Logical *p;
+  SentenceAlign *align;
 
   if (recog->process_list->next != NULL) multi = TRUE;
   else multi = FALSE;
@@ -922,21 +921,26 @@ result_pass2(Recog *recog, void *dummy)
       }
       
       /* output alignment result if exist */
-      if (s->align.filled) {
-	HMM_Logical *p;
-	int i;
-	
+      for (align = s->align; align; align = align->next) {
 	printf("=== begin forced alignment ===\n");
+	switch(align->unittype) {
+	case PER_WORD:
+	  printf("-- word alignment --\n"); break;
+	case PER_PHONEME:
+	  printf("-- phoneme alignment --\n"); break;
+	case PER_STATE:
+	  printf("-- state alignment --\n"); break;
+	}
 	printf(" id: from  to    n_score    unit\n");
 	printf(" ----------------------------------------\n");
-	for(i=0;i<s->align.num;i++) {
-	  printf("[%4d %4d]  %f  ", s->align.begin_frame[i], s->align.end_frame[i], s->align.avgscore[i]);
-	  switch(s->align.unittype) {
+	for(i=0;i<align->num;i++) {
+	  printf("[%4d %4d]  %f  ", align->begin_frame[i], align->end_frame[i], align->avgscore[i]);
+	  switch(align->unittype) {
 	  case PER_WORD:
-	    myprintf("%s\t[%s]\n", winfo->wname[s->align.w[i]], winfo->woutput[s->align.w[i]]);
+	    myprintf("%s\t[%s]\n", winfo->wname[align->w[i]], winfo->woutput[align->w[i]]);
 	    break;
 	  case PER_PHONEME:
-	    p = s->align.ph[i];
+	    p = align->ph[i];
 	    if (p->is_pseudo) {
 	      printf("{%s}\n", p->name);
 	    } else if (strmatch(p->name, p->body.defined->name)) {
@@ -946,7 +950,7 @@ result_pass2(Recog *recog, void *dummy)
 	    }
 	    break;
 	  case PER_STATE:
-	    p = s->align.ph[i];
+	    p = align->ph[i];
 	    if (p->is_pseudo) {
 	      printf("{%s}", p->name);
 	    } else if (strmatch(p->name, p->body.defined->name)) {
@@ -955,20 +959,20 @@ result_pass2(Recog *recog, void *dummy)
 	      printf("%s[%s]", p->name, p->body.defined->name);
 	    }
 	    if (r->am->hmminfo->multipath) {
-	      if (s->align.is_iwsp[i]) {
-		printf(" #%d (sp)\n", s->align.loc[i]);
+	      if (align->is_iwsp[i]) {
+		printf(" #%d (sp)\n", align->loc[i]);
 	      } else {
-		printf(" #%d\n", s->align.loc[i]);
+		printf(" #%d\n", align->loc[i]);
 	      }
 	    } else {
-	      printf(" #%d\n", s->align.loc[i]);
+	      printf(" #%d\n", align->loc[i]);
 	    }
 	    break;
 	  }
 	}
 	
-	printf("re-computed AM score: %f\n", s->align.allscore);
-	
+	printf("re-computed AM score: %f\n", align->allscore);
+
 	printf("=== end forced alignment ===\n");
       }
     }
@@ -1182,6 +1186,7 @@ print_all_gram(Recog *recog)
   MULTIGRAM *m;
   RecogProcess *r;
   boolean multi;
+  char buf[1024];
 
   if (recog->process_list->next != NULL) multi = TRUE;
   else multi = FALSE;
@@ -1189,18 +1194,40 @@ print_all_gram(Recog *recog)
   for(r=recog->process_list;r;r=r->next) {
     if (! r->live) continue;
     if (multi) printf("[#%d %s]\n", r->config->id, r->config->name);
-
+    if (r->lmtype == LM_PROB) {
+      printf("NOT A GRAMMAR-BASED LM\n");
+      continue;
+    }
     printf("[grammars]\n");
     for(m=r->lm->grammars;m;m=m->next) {
-      printf("  #%2d: [%-11s] %4d words, %3d categories, %4d nodes",
-	     m->id,
-	     m->active ? "active" : "inactive",
-	     m->winfo->num, m->dfa->term_num, m->dfa->state_num);
-      if (m->newbie) printf(" (new)");
-      if (m->hook != MULTIGRAM_DEFAULT) {
-	printf(" (next: %s)", hookstr[m->hook]);
+      buf[0] = '\0';
+      if (m->dfa) {
+	snprintf(buf, 1024, ", %3d categories, %4d nodes",
+		 m->dfa->term_num, m->dfa->state_num);
       }
-      myprintf(" \"%s\"\n", m->name);
+      if (m->newbie) strcat(buf, " (new)");
+      if (m->hook != 0) {
+	strcat(buf, " (next:");
+	if (m->hook & MULTIGRAM_DELETE) {
+	  strcat(buf, " delete");
+	}
+	if (m->hook & MULTIGRAM_ACTIVATE) {
+	  strcat(buf, " activate");
+	}
+	if (m->hook & MULTIGRAM_DEACTIVATE) {
+	  strcat(buf, " deactivate");
+	}
+	if (m->hook & MULTIGRAM_MODIFIED) {
+	  strcat(buf, " modified");
+	}
+	strcat(buf, ")");
+      }
+      myprintf("  #%2d: [%-11s] %4d words%s \"%s\"\n",
+	       m->id,
+	       m->active ? "active" : "inactive",
+	       m->winfo->num,
+	       buf,
+	       m->name);
     }
     if (r->lm->dfa != NULL) {
       printf("  Global:            %4d words, %3d categories, %4d nodes\n", r->lm->winfo->num, r->lm->dfa->term_num, r->lm->dfa->state_num);
