@@ -95,7 +95,7 @@
  * @author Akinobu LEE
  * @date   Sat Feb 12 13:20:53 2005
  *
- * $Revision: 1.8 $
+ * $Revision: 1.10 $
  * 
  */
 /*
@@ -194,6 +194,8 @@ adin_setup_param(ADIn *adin, Jconf *jconf)
 
   adin->rehash = FALSE;
 
+  adin->total_captured_len = 0;
+
   return TRUE;
 
 }
@@ -417,6 +419,8 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 	plugin_exec_adin_captured(&(a->buffer[a->bp]), cnt);
 #endif
 	callback_exec_adin(CALLBACK_ADIN_CAPTURED, recog, &(a->buffer[a->bp]), cnt);
+	/* record total number of captured samples */
+	a->total_captured_len += cnt;
       }
 
       /*************************************************/
@@ -532,7 +536,7 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
     /*********************************************************/
     i = 0;
     while (i + wstep <= imax) {
-      
+
       if (a->adin_cut_on) {
 
 	/********************/
@@ -552,12 +556,13 @@ adin_cut(int (*ad_process)(SP16 *, int, Recog *), int (*ad_check)(Recog *), Reco
 	    /*****************************************************/
 	    /* process off, trigger on: detect speech triggering */
 	    /*****************************************************/
-	    
 	    a->is_valid_data = TRUE;   /* start processing */
 	    a->nc = 0;
 #ifdef THREAD_DEBUG
 	    jlog("DEBUG: detect on\n");
 #endif
+	    /* record time */
+	    a->last_trigger_sample = a->total_captured_len - a->current_len + i + wstep - a->zc.valid_len;
 	    callback_exec(CALLBACK_EVENT_SPEECH_START, recog);
 
 	    /****************************************/
@@ -970,7 +975,6 @@ adin_thread_input_main(void *dummy)
 boolean
 adin_thread_create(Recog *recog)
 {
-  pthread_t adin_thread;	///< Thread information
   ADIn *a;
 
   a = recog->adin;
@@ -987,11 +991,11 @@ adin_thread_create(Recog *recog)
     jlog("ERROR: adin_thread_create: failed to initialize mutex\n");
     return FALSE;
   }
-  if (pthread_create(&adin_thread, NULL, (void *)adin_thread_input_main, recog) != 0) {
+  if (pthread_create(&(recog->adin->adin_thread), NULL, (void *)adin_thread_input_main, recog) != 0) {
     jlog("ERROR: adin_thread_create: failed to create AD-in thread\n");
     return FALSE;
   }
-  if (pthread_detach(adin_thread) != 0) { /* not join, run forever */
+  if (pthread_detach(recog->adin->adin_thread) != 0) { /* not join, run forever */
     jlog("ERROR: adin_thread_create: failed to detach AD-in thread\n");
     return FALSE;
   }
@@ -999,6 +1003,41 @@ adin_thread_create(Recog *recog)
   return TRUE;
 }
 
+/**
+ * <EN>
+ * Delete A/D-in thread
+ * </EN>
+ * <JA>
+ * A/D-in åɤλ
+ * </JA>
+ * @param recog [in] engine instance
+ *
+ * @callergraph
+ * @callgraph
+ */
+boolean
+adin_thread_cancel(Recog *recog)
+{
+  ADIn *a;
+  int ret;
+
+  if (recog->adin->adinthread_ended) return TRUE;
+
+  ret = pthread_cancel(recog->adin->adin_thread);
+  if (ret == 0) {
+    jlog("STAT: AD-in thread deleted\n");
+  } else {
+    if (ret == ESRCH) {
+      jlog("STAT: adin_thread_cancel: no A/D-in thread\n");
+    } else {
+      jlog("Error: adin_thread_cancel: failed to cancel A/D-in thread\n");
+      return FALSE;
+    }
+  }
+  recog->adin->adinthread_ended = TRUE;
+  return TRUE;
+}
+
 /****************************/
 /* process thread functions */
 /****************************/
