@@ -42,7 +42,7 @@
  * @author Akinobu LEE
  * @date   Tue Feb 22 17:00:45 2005
  *
- * $Revision: 1.8 $
+ * $Revision: 1.11 $
  * 
  */
 /*
@@ -583,6 +583,7 @@ find_1pass_result_word(int framelen, RecogProcess *r)
   LOGPROB maxscore;
   int i;
   TRELLIS_ATOM **idx;
+  int num;
 
   if (r->lmvar != LM_DFA_WORD) return;
 
@@ -629,13 +630,14 @@ find_1pass_result_word(int framelen, RecogProcess *r)
     /* more than one candidate is requested */
 
     /* get actual number of candidates to output */
-    r->result.sentnum = r->config->output.output_hypo_maxnum;
-    if (r->result.sentnum > bt->num[last_time]) {
-      r->result.sentnum = bt->num[last_time];
+    num = r->config->output.output_hypo_maxnum;
+    if (num > bt->num[last_time]) {
+      num = bt->num[last_time];
     }
 
     /* prepare result storage */
-    r->result.sent = (Sentence *)mymalloc(sizeof(Sentence)* r->result.sentnum);
+    result_sentence_malloc(r, num);
+    r->result.sentnum = num;
 
     /* sort by score */
     idx = (TRELLIS_ATOM **)mymalloc(sizeof(TRELLIS_ATOM *)*bt->num[last_time]);
@@ -657,8 +659,11 @@ find_1pass_result_word(int framelen, RecogProcess *r)
       s->score = tmp->backscore;
       s->score_lm = 0.0;
       s->score_am = tmp->backscore;
-      s->gram_id = 0;
-      s->align.filled = FALSE;
+      if (multigram_get_all_num(r->lm) > 0) {
+	s->gram_id = multigram_get_gram_from_wid(s->word[0], r->lm);
+      } else {
+	s->gram_id = 0;
+      }
     }
     /* free work area for sort */
     free(idx);
@@ -666,7 +671,7 @@ find_1pass_result_word(int framelen, RecogProcess *r)
   } else {			/* only max is needed */
 
     /* prepare result storage */
-    r->result.sent = (Sentence *)mymalloc(sizeof(Sentence));
+    result_sentence_malloc(r, 1);
     r->result.sentnum = 1;
     s = &(r->result.sent[0]);
     s->word_num = 1;
@@ -677,12 +682,16 @@ find_1pass_result_word(int framelen, RecogProcess *r)
     s->score = best->backscore;
     s->score_lm = 0.0;
     s->score_am = best->backscore;
-    s->gram_id = 0;
-    s->align.filled = FALSE;
+    if (multigram_get_all_num(r->lm) > 0) {
+      s->gram_id = multigram_get_gram_from_wid(s->word[0], r->lm);
+    } else {
+      s->gram_id = 0;
+    }
   }
 
   /* copy as 1st pass result */
   memcpy(&(r->result.pass1), &(r->result.sent[0]), sizeof(Sentence));
+  r->result.pass1.align = NULL;
 
   //callback_exec(CALLBACK_RESULT, r);
   //free(r->result.sent);
@@ -1713,24 +1722,31 @@ init_nodescore(HTK_Param *param, RecogProcess *r)
   }
 
   if (r->lmtype == LM_DFA && r->lmvar == LM_DFA_WORD) {
-    /* all words can appear at start */
-    for (i=0;i<wchmm->winfo->num;i++) {
-      if (wchmm->hmminfo->multipath) {
-	node = wchmm->wordbegin[i];
-      } else {
-	node = wchmm->offset[i][0];
+    /* ƥ֤ʸˡ°ñΤߵ */
+    /* only words in active grammars are allowed to be an initial words */
+    MULTIGRAM *m;
+
+    for(m = r->lm->grammars; m; m = m->next) {
+      if (m->active) {
+	for(i = m->word_begin; i < m->word_begin + m->winfo->num; i++) {
+	  if (wchmm->hmminfo->multipath) {
+	    node = wchmm->wordbegin[i];
+	  } else {
+	    node = wchmm->offset[i][0];
+	  }
+	  if (node_exist_token(d, d->tn, node, d->bos.wid) != TOKENID_UNDEFINED) continue;
+	  newid = create_token(d);
+	  new = &(d->tlist[d->tn][newid]);
+	  new->last_tre = &(d->bos);
+	  new->last_lscore = 0.0;
+	  if (wchmm->hmminfo->multipath) {
+	    new->score = 0.0;
+	  } else {
+	    new->score = outprob_style(wchmm, node, d->bos.wid, 0, param);
+	  }
+	  node_assign_token(d, node, newid);
+	}
       }
-      if (node_exist_token(d, d->tn, node, d->bos.wid) != TOKENID_UNDEFINED) continue;
-      newid = create_token(d);
-      new = &(d->tlist[d->tn][newid]);
-      new->last_tre = &(d->bos);
-      new->last_lscore = 0.0;
-      if (wchmm->hmminfo->multipath) {
-	new->score = 0.0;
-      } else {
-	new->score = outprob_style(wchmm, node, d->bos.wid, 0, param);
-      }
-      node_assign_token(d, node, newid);
     }
   }
 
