@@ -12,7 +12,7 @@
  * @author Akinobu Lee
  * @date   Wed Aug  8 14:53:53 2007
  *
- * $Revision: 1.7 $
+ * $Revision: 1.11 $
  * 
  */
 
@@ -190,6 +190,67 @@ print_mem()
 #endif
 	  
 
+/** 
+ * <EN>
+ * allocate storage of recognition alignment results.
+ *
+ * @return the new pointer
+ * </EN>
+ * <JA>
+ * 饤ȷ̤γǼ
+ *
+ * @return ݤ줿ΰؤΥݥ
+ * </JA>
+ *
+ * @callgraph
+ * @callergraph
+ * 
+ */
+SentenceAlign *
+result_align_new()
+{
+  SentenceAlign *new;
+  new = (SentenceAlign *)mymalloc(sizeof(SentenceAlign));
+  new->w = NULL;
+  new->ph = NULL;
+  new->loc = NULL;
+  new->begin_frame = NULL;
+  new->end_frame = NULL;
+  new->avgscore = NULL;
+  new->is_iwsp = NULL;
+  new->next = NULL;
+  return new;
+}
+
+/** 
+ * <EN>
+ * free storage of recognition alignment results.
+ *
+ * @param a [i/o] alignment data to be released
+ * </EN>
+ * <JA>
+ * 饤ȷ̤γǼ
+ *
+ * @param a [i/o] 륢饤ȥǡ
+ * </JA>
+ *
+ * @callgraph
+ * @callergraph
+ * 
+ */
+void
+result_align_free(SentenceAlign *a)
+{
+  if (a->w) free(a->w);
+  if (a->ph) free(a->ph);
+  if (a->loc) free(a->loc);
+  if (a->begin_frame) free(a->begin_frame);
+  if (a->end_frame) free(a->end_frame);
+  if (a->avgscore) free(a->avgscore);
+  if (a->is_iwsp) free(a->is_iwsp);
+  free(a);
+}
+
 /** 
  * <EN>
  * Allocate storage of recognition results.
@@ -210,16 +271,7 @@ result_sentence_malloc(RecogProcess *r, int num)
 {
   int i;
   r->result.sent = (Sentence *)mymalloc(sizeof(Sentence) * num);
-  for(i=0;i<num;i++) {
-    r->result.sent[i].align.filled = FALSE;
-    r->result.sent[i].align.w = NULL;
-    r->result.sent[i].align.ph = NULL;
-    r->result.sent[i].align.loc = NULL;
-    r->result.sent[i].align.begin_frame = NULL;
-    r->result.sent[i].align.end_frame = NULL;
-    r->result.sent[i].align.avgscore = NULL;
-    r->result.sent[i].align.is_iwsp = NULL;
-  }
+  for(i=0;i<num;i++) r->result.sent[i].align = NULL;
   r->result.sentnum = 0;
 }
 
@@ -240,15 +292,15 @@ void
 result_sentence_free(RecogProcess *r)
 {  
   int i;
+  SentenceAlign *a, *atmp;
   if (r->result.sent) {
     for(i=0;i<r->result.sentnum;i++) {
-      if (r->result.sent[i].align.w) free(r->result.sent[i].align.w);
-      if (r->result.sent[i].align.ph) free(r->result.sent[i].align.ph);
-      if (r->result.sent[i].align.loc) free(r->result.sent[i].align.loc);
-      if (r->result.sent[i].align.begin_frame) free(r->result.sent[i].align.begin_frame);
-      if (r->result.sent[i].align.end_frame) free(r->result.sent[i].align.end_frame);
-      if (r->result.sent[i].align.avgscore) free(r->result.sent[i].align.avgscore);
-      if (r->result.sent[i].align.is_iwsp) free(r->result.sent[i].align.is_iwsp);
+      a = r->result.sent[i].align;
+      while(a) {
+	atmp = a->next;
+	result_align_free(a);
+	a = atmp;
+      }
     }
     free(r->result.sent);
     r->result.sent = NULL;
@@ -431,7 +483,7 @@ callback_check_in_adin(Recog *recog)
  * @param recog [i/o] engine instance
  * @param file_or_dev_name [in] file or device name of the device
  * 
- * @return 0 on success, -1 on error, -2 on decice initializatino error.
+ * @return 0 on success, -1 on error, -2 on device initialization error.
  * 
  * @callgraph
  * @callergraph
@@ -478,9 +530,64 @@ j_open_stream(Recog *recog, char *file_or_dev_name)
       callback_exec(CALLBACK_STATUS_PARAM, recog);
       break;
     default:
-      jlog("ERROR: none of SP_MFC_*??\n");
+      jlog("ERROR: j_open_stream: none of SP_MFC_*??\n");
+      return -1;
+    }
+  }
+      
+  return 0;
+
+}
+
+/** 
+ * <EN>
+ * Close input stream.  The main recognition loop will be stopped after
+ * stream has been closed.
+ * </EN>
+ * <JA>
+ * ϥȥ꡼Ĥ롥ǧΥᥤ롼פĤ줿彪λ롥
+ * </JA>
+ * 
+ * @param recog [i/o] engine instance
+ * 
+ * @return 0 on success, -1 on general error, -2 on device error.
+ * 
+ * @callgraph
+ * @callergraph
+ * @ingroup engine
+ */
+int
+j_close_stream(Recog *recog)
+{
+  Jconf *jconf;
+
+  jconf = recog->jconf;
+
+  if (jconf->input.type == INPUT_WAVEFORM) {
+#ifdef HAVE_PTHREAD
+    /* close A/D-in thread here */
+    if (recog->adin->enable_thread && ! recog->adin->input_side_segment) {
+      if (adin_thread_cancel(recog) == FALSE) {
+	return -2;
+      }
+    }
+#endif
+    /* end A/D input */
+    if (adin_end(recog->adin) == FALSE) {
       return -2;
     }
+  } else {
+    switch(jconf->input.speech_input) {
+    case SP_MFCMODULE:
+      if (mfc_module_end(recog->mfcclist) == FALSE) return -2;
+      break;
+    case SP_MFCFILE:
+      /* nothing to do */
+      break;
+    default:
+      jlog("ERROR: j_close_stream: none of SP_MFC_*??\n");
+      return -1;
+    }
   }
       
   return 0;
@@ -576,6 +683,7 @@ j_recognize_stream_core(Recog *recog)
   boolean ok_p;
   boolean process_segment_last;
   boolean on_the_fly;
+  boolean pass2_p;
 
   jconf = recog->jconf;
 
@@ -1050,9 +1158,40 @@ j_recognize_stream_core(Recog *recog)
       }
     }
 
+    /* for instances with "-1pass", copy 1st pass result as final */
+    /* execute stack-decoding search */
+    /* they will be skipepd in the next pass */
+    for(r=recog->process_list;r;r=r->next) {
+      if (!r->live) continue;
+      /* skip if 1st pass was failed */
+      if (r->result.status < 0) continue;
+      /* already stored on word recognition, so skip this */
+      if (r->lmvar == LM_DFA_WORD) continue;
+      if (r->config->compute_only_1pass) {
+	if (verbose_flag) {
+	  jlog("%02d %s: \"-1pass\" specified, output 1st pass result as a final result\n", r->config->id, r->config->name);
+	}
+	/* prepare result storage */
+	result_sentence_malloc(r, 1);
+	/* finalize result when no hypothesis was obtained */
+	pass2_finalize_on_no_result(r, TRUE);
+      }
+    }
+
     /***********************************************/
     /* 2nd-pass --- forward search with heuristics */
     /***********************************************/
+    pass2_p = FALSE;
+    for(r=recog->process_list;r;r=r->next) {
+      if (!r->live) continue;
+      /* if [-1pass] is specified, skip 2nd pass */
+      if (r->config->compute_only_1pass) continue;
+      /* if search already failed on 1st pass, skip 2nd pass */
+      if (r->result.status < 0) continue;
+      pass2_p = TRUE;
+    }
+    if (pass2_p) callback_exec(CALLBACK_EVENT_PASS2_BEGIN, recog);
+
 #if !defined(PASS2_STRICT_IWCD) || defined(FIX_35_PASS2_STRICT_SCORE)    
     /* adjust trellis score not to contain outprob of the last frames */
     for(r=recog->process_list;r;r=r->next) {
@@ -1073,11 +1212,9 @@ j_recognize_stream_core(Recog *recog)
 #endif
     
     /* execute stack-decoding search */
-    callback_exec(CALLBACK_EVENT_PASS2_BEGIN, recog);
-
     for(r=recog->process_list;r;r=r->next) {
       if (!r->live) continue;
-      /* if [-1pass] is specified, skip 2nd pass */
+      /* if [-1pass] is specified, just copy from 1st pass result */
       if (r->config->compute_only_1pass) continue;
       /* if search already failed on 1st pass, skip 2nd pass */
       if (r->result.status < 0) continue;
@@ -1113,6 +1250,15 @@ j_recognize_stream_core(Recog *recog)
       }
     }
 
+    /* do forced alignment if needed */
+    for(r=recog->process_list;r;r=r->next) {
+      if (!r->live) continue;
+      /* if search failed on 2nd pass, skip this */
+      if (r->result.status < 0) continue;
+      /* do needed alignment */
+      do_alignment_all(r, r->am->mfcc->param);
+    }
+
     /* output result */
     callback_exec(CALLBACK_RESULT, recog);
 #ifdef ENABLE_PLUGIN
@@ -1123,6 +1269,8 @@ j_recognize_stream_core(Recog *recog)
     ok_p = FALSE;
     for(r=recog->process_list;r;r=r->next) {
       if (!r->live) continue;
+      if (r->config->compute_only_1pass) continue;
+      if (r->result.status < 0) continue;
       if (r->config->graph.lattice) ok_p = TRUE;
     }
     if (ok_p) callback_exec(CALLBACK_RESULT_GRAPH, recog);
@@ -1131,6 +1279,8 @@ j_recognize_stream_core(Recog *recog)
     ok_p = FALSE;
     for(r=recog->process_list;r;r=r->next) {
       if (!r->live) continue;
+      if (r->config->compute_only_1pass) continue;
+      if (r->result.status < 0) continue;
       if (r->config->graph.confnet) ok_p = TRUE;
     }
     if (ok_p) callback_exec(CALLBACK_RESULT_CONFNET, recog);
@@ -1142,7 +1292,7 @@ j_recognize_stream_core(Recog *recog)
     }
     
     /* output end of 2nd pass */
-    callback_exec(CALLBACK_EVENT_PASS2_END, recog);
+    if (pass2_p) callback_exec(CALLBACK_EVENT_PASS2_END, recog);
 
 #ifdef DEBUG_VTLN_ALPHA_TEST
     if (r->am->mfcc->para->vtln_alpha == 1.0) {
