@@ -19,7 +19,7 @@
  * @author Akinobu LEE
  * @date   Thu Feb 10 17:22:36 2005
  *
- * $Revision: 1.2 $ 
+ * $Revision: 1.4 $ 
  */
 /*
  * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
@@ -34,14 +34,41 @@
 #include <sent/stddefs.h>
 #include <sent/speech.h>
 
-/// To select speech input source
+#if defined(HAVE_ALSA_ASOUNDLIB_H) || defined(HAVE_SYS_ASOUNDLIB_H)
+#define HAS_ALSA
+#endif
+#ifdef __linux__
+#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
+#define HAS_OSS
+#endif
+#endif /* __linux__ */
+#ifdef HAVE_ESD_H
+#define HAS_ESD
+#endif
+
+/// Speech input type
+enum {
+  INPUT_WAVEFORM,
+  INPUT_VECTOR
+};
+
+/// Speech input source
 enum {
   SP_RAWFILE,			///< Wavefile
-  SP_MIC,			///< Live microphone device
+  SP_MIC,			///< Live microphone device, or plugin
   SP_ADINNET,			///< Network client (adintool etc.)
   SP_MFCFILE,			///< HTK parameter file
   SP_NETAUDIO,			///< Live NetAudio/DatLink input
-  SP_STDIN			///< Standard input
+  SP_STDIN,			///< Standard input
+  SP_MFCMODULE			///< parameter module
+};
+
+/// Input device
+enum {
+  SP_INPUT_DEFAULT,
+  SP_INPUT_ALSA,
+  SP_INPUT_OSS,
+  SP_INPUT_ESD,
 };
 
 /// Default unit size of speech input segment in bytes
@@ -122,6 +149,21 @@ boolean adin_mic_standby(int freq, void *arg);
 boolean adin_mic_begin();
 boolean adin_mic_end();
 int adin_mic_read(SP16 *buf, int sampnum);
+/* adin/adin_mic_linux_alsa.c */
+boolean adin_alsa_standby(int freq, void *arg);
+boolean adin_alsa_begin();
+boolean adin_alsa_end();
+int adin_alsa_read(SP16 *buf, int sampnum);
+/* adin/adin_mic_linux_oss.c */
+boolean adin_oss_standby(int freq, void *arg);
+boolean adin_oss_begin();
+boolean adin_oss_end();
+int adin_oss_read(SP16 *buf, int sampnum);
+/* adin/adin_esd.c */
+boolean adin_esd_standby(int freq, void *arg);
+boolean adin_esd_begin();
+boolean adin_esd_end();
+int adin_esd_read(SP16 *buf, int sampnum);
 /* adin/adin_netaudio.c  and adin/adin_na.c */
 boolean adin_netaudio_standby(int freq, void *arg);
 boolean adin_netaudio_begin();
