@@ -1,7 +1,5 @@
 /**
  * @file   htk_defs.h
- * @author Akinobu LEE
- * @date   Thu Feb 10 19:36:47 2005
  *
  * <EN>
  * @brief Symbol definitions for HTK HMM and HTK parameter
@@ -23,13 +21,16 @@
  * @sa htk_param.h
  * </JA>
  *
- * $Revision: 1.2 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 10 19:36:47 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -95,5 +96,23 @@ typedef struct {
 /// Header string to detect binary HMM file
 #define BINHMM_HEADER "JBINHMM\n"
 
+/// Header string for binary HMM file V2 (parameter embedded)
+#define BINHMM_HEADER_V2 "JBINHMMV2"
+
+/// A header qualifier string for V2: acoustic analysis parameter embedded
+#define BINHMM_HEADER_V2_EMBEDPARA 'P'
+
+/// A header qualifier string for V2: variance inversed
+#define BINHMM_HEADER_V2_VARINV 'V'
+
+/// A header qualifier string for V2: has mixture pdf macro def
+#define BINHMM_HEADER_V2_MPDFMACRO 'M'
+
+/// Maximum number of input stream
+#define MAXSTREAMNUM 50
+
+#ifdef ENABLE_MSD
+#define LZERO (-1.0E10) 		///< log(0) value of void dimension for MSD-HMM */
+#endif
 
 #endif /* __SENT_HTK_DEFS_H__ */
