@@ -21,7 +21,7 @@
  * @author Akinobu LEE
  * @date   Sat Feb 12 12:30:40 2005
  *
- * $Revision: 1.4 $
+ * $Revision: 1.6 $
  * 
  */
 /*
@@ -60,8 +60,6 @@ typedef struct _bmalloc_base {
 /* readfile.c */
 char *getl(char *, int, FILE *);
 char *getl_fp(char *, int, FILE *);
-char *getl_fd(char *, int, int);
-char *getl_sd(char *, int, int);
 char *get_line_from_stdin(char *buf, int buflen, char *prompt);
 
 /* gzfile.c */
@@ -81,9 +79,10 @@ char *mybstrdup2(char *, BMALLOC_BASE **list);
 void mybfree2(BMALLOC_BASE **list);
 
 /* mymalloc.c */
-void *mymalloc(int);
-void *myrealloc(void *, int);
-void *mycalloc(int, int);
+void *mymalloc(size_t size);
+void *mymalloc_big(size_t elsize, size_t nelem);
+void *myrealloc(void *, size_t);
+void *mycalloc(size_t, size_t);
 
 /* endian.c */
 void swap_sample_bytes(SP16 *buf, int len);
