@@ -1,7 +1,5 @@
 /**
  * @file   adin_na.c
- * @author Akinobu LEE
- * @date   Sun Feb 13 19:40:56 2005
  *
  * <JA>
  * @brief  NetAudioѤΥ֥롼
@@ -22,13 +20,16 @@
  * include header "sent/stddefs.h".
  * </EN>
  * 
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 19:40:56 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -77,7 +78,7 @@ NA_standby(int sfreq, char *server_devname)
   /* if AUDIO_DEVICE not specified, local port is used */
   port = NAOpen(server_devname);
   if (port == NULL) {
-    j_printerr("failed to open netaudio server on %s\n", server_devname);
+    jlog("Error: adin_na: failed to open netaudio server on %s\n", server_devname);
     return(FALSE);
   }
 
@@ -92,12 +93,11 @@ NA_standby(int sfreq, char *server_devname)
 
   /* open a data connection for recording */
   if (NAOpenData(port, NA_RECORD) == -1) {
-    perror("cannot open data connection");
+    jlog("Error: adin_na: failed to open data connection\n");
     return(FALSE);
   }
 
-  j_printerr("connected to netaudio server\n");
-
+  jlog("Stat: adin_na: connected to netaudio server on %s\n", server_devname);
   return(TRUE);
 }
 
