@@ -1,7 +1,5 @@
 /**
  * @file   adin_netaudio.c
- * @author Akinobu LEE
- * @date   Sun Feb 13 19:50:55 2005
  *
  * <JA>
  * @brief  ͥåȥϡNetAudio/DatLink Фβ
@@ -25,13 +23,16 @@
  * The actual procedure are defined in adin_na.c.
  * </EN>
  *
- * $Revision: 1.3 $
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 19:50:55 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -68,10 +69,9 @@ adin_netaudio_standby(int sfreq, void *arg)
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_netaudio_start()
+adin_netaudio_begin()
 {
   NA_start();
-  j_printerr("<<< please speak >>>\n");
   return(TRUE);
 }
 
@@ -81,7 +81,7 @@ adin_netaudio_start()
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_netaudio_stop()
+adin_netaudio_end()
 {
   NA_stop();
   return(TRUE);
@@ -105,7 +105,7 @@ adin_netaudio_read(SP16 *buf, int sampnum)
   int cnt;
   cnt = NA_read(buf, sampnum);
   if (cnt < 0) {
-    j_printerr("Error: failed to read sample\n");
+    jlog("Error: adin_na: failed to read sample\n");
     return(-2);			/* return negative on error */
   }
   return(cnt);
