@@ -1,7 +1,5 @@
 /**
  * @file   adin_portaudio.c
- * @author Akinobu LEE
- * @date   Mon Feb 14 12:03:48 2005
  * 
  * <JA>
  * @brief  ޥ (portaudio饤֥)
@@ -43,12 +41,15 @@
  * @sa http://www.portaudio.com/
  * </EN>
  *
- * $Revision: 1.4 $
+ * @author Akinobu LEE
+ * @date   Mon Feb 14 12:03:48 2005
+ *
+ * $Revision: 1.2 $
  * 
  */
 /*
  * Copyright (c) 2004-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -62,6 +63,14 @@
 
 #undef DDEBUG
 
+/**
+ * Maximum Data fragment Length in msec.  Input can be delayed to this time.
+ * You can override this value by specifying environment valuable
+ * "LATENCY_MSEC".
+ * 
+ */
+#define MAX_FRAGMENT_MSEC 128
+
 /* temporal buffer */
 static SP16 *speech;		///< cycle buffer for incoming speech data
 static int current;		///< writing point
@@ -151,14 +160,19 @@ boolean
 adin_mic_standby(int sfreq, void *dummy)
 {
   PaError err;
+  int frames_per_buffer, num_buffer;
+  int latency;
+  char *p;
 
   /* set cycle buffer length */
   cycle_buffer_len = INPUT_DELAY_SEC * sfreq;
-  j_printerr("Audio cycle buffer length: %d bytes\n", cycle_buffer_len * sizeof(SP16));
+  jlog("Stat: adin_portaudio: INPUT_DELAY_SEC = %d\n", INPUT_DELAY_SEC);
+  jlog("Stat: adin_portaudio: audio cycle buffer length = %d bytes\n", cycle_buffer_len * sizeof(SP16));
 
   /* for safety... */
   if (sizeof(SP16) != paInt16) {
-    j_error("SP16 != paInt16\n");
+    jlog("Error: adin_portaudio: SP16 != paInt16 !!\n");
+    return FALSE;
   }
 
   /* allocate and init */
@@ -166,17 +180,35 @@ adin_mic_standby(int sfreq, void *dummy)
   speech = (SP16 *)mymalloc(sizeof(SP16) * cycle_buffer_len);
   buffer_overflowed = FALSE;
 
+  /* set buffer parameter*/
+  if ((p = getenv("LATENCY_MSEC")) != NULL) {
+    latency = atoi(p);
+    jlog("Stat: adin_portaudio: set latency to %d msec (obtained from LATENCY_MSEC)\n", latency);
+  } else {
+    latency = MAX_FRAGMENT_MSEC;
+    jlog("Stat: adin_portaudio: set latency to %d msec\n", latency);
+  }
+  frames_per_buffer = 256;
+  num_buffer = sfreq * latency / (frames_per_buffer * 1000);
+  jlog("Stat: adin_portaudio: framesPerBuffer=%d, NumBuffers(guess)=%d (%d)\n",
+	   frames_per_buffer, num_buffer, 
+	   Pa_GetMinNumBuffers(frames_per_buffer, sfreq));
+  jlog("Stat: adin_portaudio: audio I/O Latency = %d msec (data fragment = %d frames)\n",
+	   (frames_per_buffer * num_buffer) * 1000 / sfreq, 
+	   (frames_per_buffer * num_buffer));
+
   /* initialize device and open stream */
   err = Pa_Initialize();
   if (err != paNoError) {
-    j_printerr("Error: %s\n", Pa_GetErrorText(err));
+    jlog("Error: adin_portaudio: failed to initialize: %s\n", Pa_GetErrorText(err));
     return(FALSE);
   }
-  
-  err = Pa_OpenDefaultStream(&stream, 1, 0, paInt16, sfreq, 128, 0, 
+
+  err = Pa_OpenDefaultStream(&stream, 1, 0, paInt16, sfreq, 
+			     frames_per_buffer, num_buffer, 
 			     Callback, NULL);
   if (err != paNoError) {
-    j_printerr("Error: %s\n", Pa_GetErrorText(err));
+    jlog("Error: adin_portaudio: error in opening stream: %s\n", Pa_GetErrorText(err));
     return(FALSE);
   }
 
@@ -189,14 +221,14 @@ adin_mic_standby(int sfreq, void *dummy)
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_start()
+adin_mic_begin()
 {
   PaError err;
 
   /* start stream */
   err = Pa_StartStream(stream);
   if (err != paNoError) {
-    j_printerr("Error: %s\n", Pa_GetErrorText(err));
+    jlog("Error: adin_portaudio: failed to begin stream: %s\n", Pa_GetErrorText(err));
     return(FALSE);
   }
   
@@ -209,14 +241,14 @@ adin_mic_start()
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_stop()
+adin_mic_end()
 {
   PaError err;
 
   /* stop stream */
   err = Pa_StopStream(stream);
   if (err != paNoError) {
-    j_printerr("Error: %s\n", Pa_GetErrorText(err));
+    jlog("Error: adin_portaudio: failed to stop stream: %s\n", Pa_GetErrorText(err));
     return(FALSE);
   }
   
@@ -243,7 +275,7 @@ adin_mic_read(SP16 *buf, int sampnum)
   int len;
 
   if (buffer_overflowed) {
-    j_printerr("Error: input buffer OVERFLOW, increase INPUT_DELAY_SEC in sent/speech.h\n");
+    jlog("Error: adin_portaudio: input buffer OVERFLOW, increase INPUT_DELAY_SEC in sent/speech.h\n");
     buffer_overflowed = FALSE;
   }
 
@@ -251,7 +283,7 @@ adin_mic_read(SP16 *buf, int sampnum)
 #ifdef DDEBUG
     printf("process  : current == processed: %d: wait\n", current);
 #endif
-    Pa_Sleep(50);		/* wait for 100ms, till some input comes */
+    Pa_Sleep(30); /* wait till some input comes */
   }
 
   current_local = current;
