@@ -1,7 +1,5 @@
 /**
  * @file   wrwav.c
- * @author Akinobu LEE
- * @date   Tue Feb 15 01:02:18 2005
  *
  * <JA>
  * @brief  ȷǡ WAV ե¸
@@ -10,13 +8,16 @@
  * @brief  Write waveform data to WAV file
  * </EN>
  *
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 01:02:18 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -53,7 +54,7 @@ mywrite(void *buf, size_t unitbyte, int unitnum, FILE *fp)
 }
 
 /// Function macro to mywrite() with error handling.
-#define MYWRITE(A,B,C,D)  if (!mywrite(A, B, C, D)) {j_printerr("wrwav_open: write failed\n"); fclose_writefile(fp); return NULL;}
+#define MYWRITE(A,B,C,D)  if (!mywrite(A, B, C, D)) {jlog("Error: wrwav: failed to write wave data\n"); fclose_writefile(fp); return NULL;}
 
 /* open/create a WAVE file for writing, and write header */
 /* return file pointer, NULL on failure */
@@ -150,21 +151,21 @@ wrwav_close(FILE *fp)
   /* overwrite data length after recording */
   /* at 5-8(len+36), 41-44 (len) */
   if (fseek(fp, 40, SEEK_SET) != 0) { /* error */
-    perror("wrwav_close");
+    jlog("Error: wrwav: failed to seek for header\n");
     return(FALSE);
   }
   i = totallen * sizeof(SP16);
   if (!mywrite(&i, 4, 1, fp)) {
-    j_printerr("wrwav_close: failed to re-write header\n");
+    jlog("Error: wrwav: failed to re-write header\n");
     return(FALSE);
   }
   if (fseek(fp, 4, SEEK_SET) != 0) { /* error */
-    perror("wrwav_close");
+    jlog("Error: wrwav: failed to seek for header\n");
     return(FALSE);
   }
   i = totallen * sizeof(SP16) + 36;
   if (!mywrite(&i, 4, 1, fp)) {
-    j_printerr("wrwav_close: failed to re-write header\n");
+    jlog("Error: wrwav: failed to re-write header\n");
     return(FALSE);
   }
   
