@@ -20,7 +20,7 @@
  * @author Akinobu LEE
  * @date   Wed Feb 16 16:52:24 2005
  *
- * $Revision: 1.7 $
+ * $Revision: 1.15 $
  * 
  */
 /*
@@ -30,7 +30,7 @@
  * All rights reserved
  */
 
-/* $Id: ngram_read_arpa.c,v 1.7 2008/03/20 09:08:04 sumomo Exp $ */
+/* $Id: ngram_read_arpa.c,v 1.15 2009/02/10 08:15:48 sumomo Exp $ */
 
 /* words should be alphabetically sorted */
 
@@ -55,7 +55,7 @@ get_total_info(FILE *fp, NNID num[])
   char *p;
   int n;
   int maxn;
-  long entry_num;
+  unsigned long entry_num;
 
   maxn = 0;
 
@@ -70,7 +70,8 @@ get_total_info(FILE *fp, NNID num[])
 	return -1;
       }
       p = strtok(NULL, "=");
-      entry_num = atol(p);
+      //entry_num = atol(p);
+      sscanf(p, "%lu", &entry_num);
       /* check maximum number */
       if (entry_num > NNID_MAX) {
 	jlog("Error: too big %d-gram (exceeds %d bit)\n", n, sizeof(NNID) * 8);
@@ -118,8 +119,8 @@ set_unigram(FILE *fp, NGRAM_INFO *ndata)
   t->num = NULL;
   t->bgnlistlen = 0;
   t->nnid2wid = NULL;
-  t->prob = (LOGPROB *)mymalloc(sizeof(LOGPROB) * t->totalnum);
-  t->bo_wt = (LOGPROB *)mymalloc(sizeof(LOGPROB) * t->totalnum);
+  t->prob = (LOGPROB *)mymalloc_big(sizeof(LOGPROB), t->totalnum);
+  t->bo_wt = (LOGPROB *)mymalloc_big(sizeof(LOGPROB), t->totalnum);
   t->context_num = t->totalnum;
   t->nnid2ctid_upper = NULL;
   t->nnid2ctid_lower = NULL;
@@ -128,20 +129,20 @@ set_unigram(FILE *fp, NGRAM_INFO *ndata)
   
   while (getl(buf, sizeof(buf), fp) != NULL && buf[0] != '\\') {
     if ((p = strtok(buf, DELM)) == NULL) {
-      jlog("Error: ngram_read_arpa: LR 1-gram: failed to parse, corrupted or invalid data?\n");
+      jlog("Error: ngram_read_arpa: 1-gram: failed to parse, corrupted or invalid data?\n");
       return FALSE;
     }
     prob = (LOGPROB)atof(p);
     if ((p = strtok(NULL, DELM)) == NULL) {
-      jlog("Error: ngram_read_arpa: LR 1-gram: failed to parse, corrupted or invalid data?\n");
+      jlog("Error: ngram_read_arpa: 1-gram: failed to parse, corrupted or invalid data?\n");
       return FALSE;
     }
     name = strcpy((char *)mymalloc(strlen(p)+1), p);
     if ((p = strtok(NULL, DELM)) == NULL) {
-      jlog("Error: ngram_read_arpa: LR 1-gram: failed to parse, corrupted or invalid data?\n");
-      return FALSE;
+      bo_wt = 0.0;
+    } else {
+      bo_wt = (LOGPROB)atof(p);
     }
-    bo_wt = (LOGPROB)atof(p);
 
     /* register word entry name */
     ndata->wname[nid] = name;
@@ -202,7 +203,7 @@ add_unigram(FILE *fp, NGRAM_INFO *ndata)
   boolean ok_p = TRUE;
   boolean mismatched = FALSE;
 
-  ndata->bo_wt_1 = (LOGPROB *)mymalloc(sizeof(LOGPROB) * ndata->max_word_num);
+  ndata->bo_wt_1 = (LOGPROB *)mymalloc_big(sizeof(LOGPROB), ndata->max_word_num);
 
   read_word_num = 0;
   while (getl(buf, sizeof(buf), fp) != NULL && buf[0] != '\\') {
@@ -217,11 +218,11 @@ add_unigram(FILE *fp, NGRAM_INFO *ndata)
     }
     name = strcpy((char *)mymalloc(strlen(p)+1), p);
     if ((p = strtok(NULL, DELM)) == NULL) {
-      jlog("Error: ngram_read_arpa: RL 1-gram: failed to parse, corrupted or invalid data?\n");
-      return FALSE;
+      bo_wt = 0.0;
+    } else {
+      bo_wt = (LOGPROB)atof(p);
     }
-    bo_wt = (LOGPROB)atof(p);
-  
+
     /* add bo_wt_rl to existing 1-gram entry */
     nid = ngram_lookup_word(ndata, name);
     if (nid == WORD_INVALID) {
@@ -250,9 +251,8 @@ add_unigram(FILE *fp, NGRAM_INFO *ndata)
 }
 
 /** 
- * Read reverse 2-gram data from RL 3-gram file, and set RL 2-gram
- * probabilities and back-off values for RL 3-gram to the corresponding
- * LR 2-gram data.
+ * Read forward 2-gram data and set the LR 2-gram probabilities to the
+ * already loaded RL N-gram.
  * 
  * @param fp [in] file pointer
  * @param ndata [i/o] N-gram to set the read data.
@@ -262,17 +262,17 @@ add_bigram(FILE *fp, NGRAM_INFO *ndata)
 {
   WORD_ID w[2], wtmp;
   LOGPROB prob;
-  long bi_count = 0;
+  NNID bi_count = 0;
   NNID n2;
   boolean ok_p = TRUE;
   char *s;
 
-  ndata->p_2 = (LOGPROB *)mymalloc(sizeof(LOGPROB)*ndata->d[1].totalnum);
+  ndata->p_2 = (LOGPROB *)mymalloc_big(sizeof(LOGPROB), ndata->d[1].totalnum);
 
   while (getl(buf, sizeof(buf), fp) != NULL && buf[0] != '\\') {
     strcpy(pbuf, buf);
     if ( ++bi_count % 100000 == 0) {
-      jlog("Stat: ngram_read_arpa: 2-gram read %d (%d%%)\n", bi_count, bi_count * 100 / ndata->d[1].totalnum);
+      jlog("Stat: ngram_read_arpa: 2-gram read %lu (%d%%)\n", bi_count, bi_count * 100 / ndata->d[1].totalnum);
     }
     if ((s = strtok(buf, DELM)) == NULL) {
       jlog("Error: ngram_read_arpa: 2-gram: failed to parse, corrupted or invalid data?\n");
@@ -284,8 +284,8 @@ add_bigram(FILE *fp, NGRAM_INFO *ndata)
       return FALSE;
     }
     w[0] = ngram_lookup_word(ndata, s);
-    if (w[1] == WORD_INVALID) {
-      jlog("Error: ngram_read_arpa: 2-gram #%d: \"%s\": \"%s\" not exist in 1-gram\n", n2+1, pbuf, s);
+    if (w[0] == WORD_INVALID) {
+      jlog("Error: ngram_read_arpa: 2-gram #%lu: \"%s\": \"%s\" not exist in 1-gram\n", bi_count, pbuf, s);
       ok_p = FALSE;
       continue;
     }
@@ -294,8 +294,8 @@ add_bigram(FILE *fp, NGRAM_INFO *ndata)
       return FALSE;
     }
     w[1] = ngram_lookup_word(ndata, s);
-    if (w[0] == WORD_INVALID) {
-      jlog("Error: ngram_read_arpa: 2-gram #%d: \"%s\": \"%s\" not exist in 1-gram\n", n2+1, pbuf, s);
+    if (w[1] == WORD_INVALID) {
+      jlog("Error: ngram_read_arpa: 2-gram #%lu: \"%s\": \"%s\" not exist in 1-gram\n", bi_count, pbuf, s);
       ok_p = FALSE;
       continue;
     }
@@ -314,7 +314,7 @@ add_bigram(FILE *fp, NGRAM_INFO *ndata)
   }
 
   if (ok_p == TRUE) {
-    jlog("Stat: ngram_read_arpa: 2-gram read %d end\n", bi_count);
+    jlog("Stat: ngram_read_arpa: 2-gram read %lu end\n", bi_count);
   }
 
   return ok_p;
@@ -352,26 +352,26 @@ set_ngram(FILE *fp, NGRAM_INFO *ndata, int n)
   /* initialize pointer storage to access from (N-1)-gram */
   t->bgnlistlen = tprev->context_num;
   if (t->is24bit) {
-    t->bgn_upper = (NNID_UPPER *)mymalloc(sizeof(NNID_UPPER) * t->bgnlistlen);
-    t->bgn_lower = (NNID_LOWER *)mymalloc(sizeof(NNID_LOWER) * t->bgnlistlen);
+    t->bgn_upper = (NNID_UPPER *)mymalloc_big(sizeof(NNID_UPPER), t->bgnlistlen);
+    t->bgn_lower = (NNID_LOWER *)mymalloc_big(sizeof(NNID_LOWER), t->bgnlistlen);
     for(i = 0; i < t->bgnlistlen; i++) {    
       t->bgn_upper[i] = NNID_INVALID_UPPER;
       t->bgn_lower[i] = 0;
     }
   } else {
-    t->bgn = (NNID *)mymalloc(sizeof(NNID) * t->bgnlistlen);
+    t->bgn = (NNID *)mymalloc_big(sizeof(NNID), t->bgnlistlen);
     for(i = 0;i < t->bgnlistlen; i++) {
       t->bgn[i] = NNID_INVALID;
     }
   }
-  t->num = (WORD_ID *)mymalloc(sizeof(WORD_ID) * t->bgnlistlen);
+  t->num = (WORD_ID *)mymalloc_big(sizeof(WORD_ID), t->bgnlistlen);
   for(i = 0; i < t->bgnlistlen; i++) {
     t->num[i] = 0;
   }
 
   /* allocate data area */
-  t->nnid2wid = (WORD_ID *)mymalloc(sizeof(WORD_ID) * t->totalnum);
-  t->prob = (LOGPROB *)mymalloc(sizeof(LOGPROB) * t->totalnum);
+  t->nnid2wid = (WORD_ID *)mymalloc_big(sizeof(WORD_ID), t->totalnum);
+  t->prob = (LOGPROB *)mymalloc_big(sizeof(LOGPROB), t->totalnum);
   t->bo_wt = NULL;
   t->nnid2ctid_upper = NULL;
   t->nnid2ctid_lower = NULL;
@@ -380,7 +380,7 @@ set_ngram(FILE *fp, NGRAM_INFO *ndata, int n)
   cid = cid_last = NNID_INVALID;
   for(i=0;i<n;i++) w_last[i] = WORD_INVALID;
 
-  /* read in 2-gram */
+  /* read in N-gram */
   for (;;) {
     if (getl(buf, sizeof(buf), fp) == NULL || buf[0] == '\\') break;
     strcpy(pbuf, buf);
@@ -388,7 +388,7 @@ set_ngram(FILE *fp, NGRAM_INFO *ndata, int n)
       jlog("Stat: ngram_read_arpa: %d-gram read %d (%d%%)\n", n, nnid, nnid * 100 / t->totalnum);
     }
 
-    /* 2-gram probability */
+    /* N-gram probability */
     if ((s = strtok(buf, DELM)) == NULL) {
       jlog("Error: ngram_read_arpa: %d-gram: failed to parse, corrupted or invalid data?\n", n);
       return FALSE;
@@ -472,9 +472,12 @@ set_ngram(FILE *fp, NGRAM_INFO *ndata, int n)
     if ((s = strtok(NULL, DELM)) != NULL) {
       bowt = (LOGPROB) atof(s);
       if (t->bo_wt == NULL) {
-	t->bo_wt = (LOGPROB *)mymalloc(sizeof(LOGPROB) * t->totalnum);
+	t->bo_wt = (LOGPROB *)mymalloc_big(sizeof(LOGPROB), t->totalnum);
+	for(i=0;i<nnid;i++) t->bo_wt[i] = 0.0;
       }
       t->bo_wt[nnid] = bowt;
+    } else {
+      if (t->bo_wt != NULL) t->bo_wt[nnid] = 0.0;
     }
 
     /* store the entry info */
@@ -552,7 +555,7 @@ ngram_read_arpa(FILE *fp, NGRAM_INFO *ndata, boolean addition)
     /* check if the numbers are the same with already read n-gram */
     for(i=0;i<2;i++) {
       if (ndata->d[i].totalnum != num[i]) {
-	jlog("Warning: ngram_read_arpa: %d-gram total num differ between forward N-gram and backward N-gram, may cause some error\n", n+1);
+	jlog("Warning: ngram_read_arpa: %d-gram total num differ between forward N-gram and backward N-gram, may cause some error\n", i+1);
       }
     }
     /* read additional 1-gram data */
@@ -650,9 +653,42 @@ ngram_read_arpa(FILE *fp, NGRAM_INFO *ndata, boolean addition)
       }
     }
     
-    /* set unknown (=OOV) word id */
-    set_unknown_id(ndata);
-    
+    /* swap <s> and </s> for backward SRILM N-gram */
+    if (ndata->dir == DIR_RL) {
+      WORD_ID bos, eos;
+      char *p;
+      bos = ngram_lookup_word(ndata, BEGIN_WORD_DEFAULT);
+      eos = ngram_lookup_word(ndata, END_WORD_DEFAULT);
+      if (!ndata->bos_eos_swap) {
+	/* check */
+	if (bos != WORD_INVALID && eos != WORD_INVALID && ndata->d[0].prob[bos] == -99) {
+	  jlog("Stat: \"P(%s) = -99\" in reverse N-gram, may be trained by SRILM\n", BEGIN_WORD_DEFAULT);
+	  jlog("Stat: going to swap \"%s\" and \"%s\"\n", BEGIN_WORD_DEFAULT, END_WORD_DEFAULT);
+	  ndata->bos_eos_swap = TRUE;
+	}
+      }
+      if (ndata->bos_eos_swap) {
+	if (bos == WORD_INVALID) {
+	  jlog("Error: ngram_read_arpa: try to swap bos/eos but \"%s\" not found in N-gram\n", BEGIN_WORD_DEFAULT);
+	}
+	if (eos == WORD_INVALID) {
+	  jlog("Error: ngram_read_arpa: try to swap bos/eos but \"%s\" not found in N-gram\n", END_WORD_DEFAULT);
+	}
+	if (bos == WORD_INVALID || eos == WORD_INVALID) {
+	  return FALSE;
+	}
+	/* do swap */
+	jlog("Stat: ngram_read_arpa: swap \"%s\" and \"%s\" at backward N-gram\n", BEGIN_WORD_DEFAULT, END_WORD_DEFAULT);
+	/* swap name buffer */
+	p = ndata->wname[bos];
+	ndata->wname[bos] = ndata->wname[eos];
+	ndata->wname[eos] = p;
+	/* replace index */
+	ptree_replace_data(BEGIN_WORD_DEFAULT, eos, ndata->root);
+	ptree_replace_data(END_WORD_DEFAULT, bos, ndata->root);
+      }
+    }
+
   }
     
 #ifdef CLASS_NGRAM
