@@ -1,7 +1,5 @@
 /**
  * @file   voca_lookup.c
- * @author Akinobu LEE
- * @date   Fri Feb 18 21:24:01 2005
  * 
  * <JA>
  * @brief  ñ켭ñθ
@@ -17,13 +15,16 @@
  * "#number".
  * </EN>
  * 
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 21:24:01 2005
+ *
  * $Revision: 1.2 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -79,7 +80,7 @@ voca_lookup_wid(char *keyword, WORD_INFO *winfo)
 	if (found == WORD_INVALID) {
 	  found = i;
 	} else {
-	  j_printerr("Warning: several \"%s\" found in dictionary, use the first one..\n");
+	  jlog("Warning: voca_lookup: several \"%s\" found in dictionary, use the first one..\n");
 	  break;
 	}
       }
@@ -90,7 +91,7 @@ voca_lookup_wid(char *keyword, WORD_INFO *winfo)
 	if (found == WORD_INVALID) {
 	  found = i;
 	} else {
-	  j_printerr("Warning: several \"%s\" found in dictionary, use the first one..\n");
+	  jlog("Warning: voca_lookup: several \"%s\" found in dictionary, use the first one..\n");
 	  break;
 	}
       }
@@ -130,7 +131,7 @@ new_str2wordseq(WORD_INFO *winfo, char *s, int *len_return)
     }
     if ((wseq[num] = voca_lookup_wid(p, winfo)) == WORD_INVALID) {
       /* not found */
-      j_printerr("word \"%s\" not found in dict\n", p);
+      jlog("Error: voca_lookup: word \"%s\" not found in dict\n", p);
       free(wseq);
       return NULL;
     }
