@@ -1,20 +1,21 @@
 #!/bin/sh
 #
-# Build all binaries under directory "build-bin".
+# Build all files under directory "build".
 #
 # This should be invoked at "..", top of the source archive.
 #
 # argument: any configure options except "--enable-setup=..." is allowed.
 # 
-JULIUS_VERSION=4.1
+JULIUS_VERSION=4.1.2
 
 ######################################################################
 
 mkdir build
 dir=`pwd`
+defconf="--without-sndfile"
 
 # make julius and other tools with default setting
-./configure --prefix=${dir}/build $*
+./configure --prefix=${dir}/build ${defconf} $*
 make
 make install
 
@@ -26,7 +27,7 @@ make install.bin INSTALLTARGET=julius-${JULIUS_VERSION}
 # standard
 cd ../libjulius
 make distclean
-./configure --prefix=${dir}/build --enable-setup=standard $*
+./configure --prefix=${dir}/build ${defconf} --enable-setup=standard $*
 make
 cd ../julius
 make clean
@@ -35,7 +36,7 @@ make install.bin INSTALLTARGET=julius-${JULIUS_VERSION}-std
 # GMM-VAD
 cd ../libjulius
 make distclean
-./configure --prefix=${dir}/build --enable-gmm-vad $*
+./configure --prefix=${dir}/build ${defconf} --enable-gmm-vad $*
 make
 cd ../julius
 make clean
@@ -44,7 +45,7 @@ make install.bin INSTALLTARGET=julius-${JULIUS_VERSION}-gmm-vad
 # Decoder-VAD
 cd ../libjulius
 make distclean
-./configure --prefix=${dir}/build --enable-decoder-vad --enable-power-reject $*
+./configure --prefix=${dir}/build ${defconf} --enable-decoder-vad --enable-power-reject $*
 make
 cd ../julius
 make clean
